/* Miscellaneous functions for xyplot
 * Copyright (C) 1994 David Deaven
 *
 * This file is part of xyplot.
 *
 * xyplot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * xyplot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with xyplot; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: misc.c,v 1.3 1995/10/10 15:03:39 deaven Exp $
 */

#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <time.h>
#include "dmd/memory.h"
#include "dmd/tools.h"
#include "xyplot.h"

#include "site.h"

extern double xmin, xmax, ymin, ymax;

/* Make PostScript instructions to generate
 * tick marks.
 */
void MakeTicks(FILE *fout, int n, double min, double stp, double max, char c)
{
  int i,l,m=0;
  double f;
  static char buffer[32];
  f=min;
  for(i=0;i<n;++i) {
    sprintf(buffer,"%.6f",f);
    l=strlen(buffer);
    while(buffer[--l]=='0');
    if(buffer[l]=='.')--l;
    if(i==0)m=l;
    else if(l>m)m=l;
    f+=stp;
  }
  f=min;
  for(i=0;i<n;++i) {
    sprintf(buffer,"%f",f); buffer[m+1]='\0';
    if(buffer[m]=='.') buffer[m]='\0';
    fprintf(fout, "(%s) %f 1 %ctick\n", buffer, f, c);
    f+=stp;
    if(f>max)break;
  }
}

char *DefaultLabel(int err, int col)
{
      static char buffer[32];
      if(err>0)
	sprintf(buffer, "col %d +/- col %d", col, err);
      else
	sprintf(buffer, "col %d", col);
      return mystrdup(buffer);
}

/* adjust a data point so that it does not exceed the
 * plot boundary.
 */
void fixpoint(double *x, double *y, double m)
{
	  if(m < 1.e10)
	  {
	    if(*x < xmin) 
	    {
	      *y -= m * (*x - xmin);
	      *x = xmin;
	    }
	    if(*x > xmax)
	    {
	      *y -= m * (*x - xmax);
	      *x = xmax;
	    }
	  }
	  if(m > 1.e-10)
	  {
	    if(*y < ymin)
	    {
	      *x -= (*y - ymin) / m;
	      *y = ymin;
	    }
	    if(*y > ymax)
	    {
	      *x -= (*y - ymax) / m;
	      *y = ymax;
	    }
	  }
}

static void getcolumn(char *s, int *c)
{
  if( *s == '%' )
    *c = XY_LAST_COLUMN;
  else
    XYintscan(s, c);
}

/* Read a column number or numbers and optional error bar column
 * number from a string.
 */
void ScanCol(char *s, int *data, int *datamax, int *error)
{
  char *pc, *pm;
  *data = *datamax = *error = XY_NO_COLUMN;
  if((pc=strchr(s,':'))!=NULL)
  {
    /* error bar specified */
    pc[0]='\0';
    getcolumn(pc+1, error);
  }
  if((pm=strchr(s,'-'))!=NULL)
  {
    /* max range specified */
    pm[0] = '\0';
    getcolumn(pm+1, datamax);
  }

  getcolumn(s, data);

  if(pm != NULL) pm[0]='-';
  if(pc != NULL) pc[0]=':';
}

/* copy a file to the stream f */
int finsert(FILE *f, char *fn)
{
  FILE *fin;
  char c;
  int i;
  if((fin=fopen_path(xyplot_path, fn, "r"))==NULL)return -1;
  while((i=fgetc(fin))!=EOF)fputc(c=i,f);
  fclose(fin);
  return 0;
}

/* return time stamp string */
char *GetTime()
{
  time_t tloc;
  char *s;
  int i;
  (void)time(&tloc);
  s=mystrdup(ctime(&tloc));
  i=strlen(s)-1; while(s[i]==' ' || s[i]=='\n')--i;
  s[i+1]='\0';
  return s;
}

/* UNIX plot command eats 4-byte coordinates */
void putUnix(FILE *f, double x, double y)
{
  int i,j;
  x=UNIX_SPACE*(x-xmin)/(xmax-xmin);
  y=UNIX_SPACE*(y-ymin)/(ymax-ymin);
  i=x;
  j=y;
  fputc(i&0xff, f);
  fputc((i&0xff00) >> 8, f); 
  fputc(j&0xff, f);
  fputc((j&0xff00) >> 8, f); 
}

void setUnix(FILE *f, double x0, double x1, double y0, double y1)
{
  double dx=(x1-x0)/20.0;
  double dy=(y1-y0)/20.0;
  fputc('e', f);
  fputc('s', f); putUnix(f, x0-dx,y0-dy); putUnix(f, x1+dx,y1+dy);
}

