/* scanrange.c -- scan data ranges in an xyplot(1) format file
 * Copyright (C) 1994 David Deaven
 *
 * This file is part of xyplot.
 *
 * xyplot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * xyplot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with xyplot; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: scanrange.c,v 1.2 1995/09/05 13:59:38 deaven Exp $
 */
#include <stdio.h>
#include "xyplot.h"

#include "site.h"

/* Scan the file and determine the range of x and y values.
   If error bars are specified, consider x+xerr as max, x-xerr as min.
 */
int XYScanRange(XYFileInfo *fi)
{
  int r;
  double x,y,xerr,yerr;
  double xmin,xmax,ymin,ymax;
  if((r = XYStartStream(fi))) return r;
  xmin=ymin=PS_BIG; xmax=ymax=-PS_BIG;
  while(XYReadStream(fi,&x,&y,&xerr,&yerr)!=EOF) {
    if((x-xerr)<xmin)xmin=x-xerr;
    if((x+xerr)>xmax)xmax=x+xerr;
    if((y-yerr)<ymin)ymin=y-yerr;
    if((y+yerr)>ymax)ymax=y+yerr;
  }
  fi->xmin=xmin;
  fi->xmax=xmax;
  fi->ymin=ymin;
  fi->ymax=ymax;

  return 0;
}

