/* Calculate tick marker spacing for xyplot
 * Copyright (C) 1994 David Deaven
 *
 * This file is part of xyplot.
 *
 * xyplot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * xyplot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with xyplot; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: ticker.c,v 1.2 1995/09/05 13:59:38 deaven Exp $
 */
#include <stdio.h>
#include <math.h>
#include "xyplot.h"

/* Determine a nice set of axis labels for a given plot range.
   Return actual number of tick marks.
   *umax > *umin regardless of min, max.
 */
int ticker(double min, double max, int ntick, int lock, 
	double *umin, double *ustp, double *umax)
{
  double d,ten=10.0;
  int np,nmin,nmax,nt,n,ninc;
/* d = total distance to cover */
  d=max-min;
  if(d<0.0)
  {
    max=min;
    min+=d;
    d*=-1.0;
  }
  else if(d==0.0) /* return something which might be useful */
  {
    if(max==0.0)
    {
      min=-1.0;
      max=1.0;
      d=max-min;
    }
    else
    {
      n=ntick/2;
      d = fabs(max);
      min-=d*n;
      max+=d*n;
      d=max-min;
    }
  }
/* change to scientific notation, base ten */
  np=log10(d); if(d<1.0)np-=1;
  d*=pow(ten,(double) -np); /* d is now in the interval [1.0, 10.0] */
/* choose which increment to use */
  nt=-log10((double) ntick)-1; ninc=1;
  while((n=d/(ninc*pow(ten,(double) nt))+1)>ntick) {
    switch(ninc) {
      case 1: ninc=2; break;
      case 2: ninc=5; break;
      case 5: ninc=1; ++nt; break;
      default: 
	fprintf(stderr,"ticker(): unexpected val=%d\n",ninc);
	exit(1);
    }
  }
  (*ustp)=ninc*pow(ten,(double) np+nt);
  nmin=floor(min/(*ustp)); (*umin)=nmin*(*ustp);
  nmax=ceil(max/(*ustp)); (*umax)=nmax*(*ustp);
  if(lock) {
    (*umin)=min;
    (*umax)=max;
  }
  return nmax-nmin+1;
}

