/* xyplot header file
 * Copyright (C) 1994 David Deaven
 *
 * This file is part of xyplot.
 *
 * xyplot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * xyplot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with xyplot; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: xyplot.h,v 1.5 1996/03/16 00:15:03 deaven Exp $
 */
#ifndef _XYPLOT_H_
#define _XYPLOT_H_

#include "dmd/spline.h"

#define XY_NULL  2.e30		/* unused number to flag null entries */
#define XY_NO_COLUMN -1		/* no column spec */
#define XY_LAST_COLUMN -387	/* last available column spec */

/* Each file has some definitions */

struct finfo {
  char *name;			/* filename */
  char *label,*xlabel,*ylabel;	/* column labels */
  double xmin,xmax,ymin,ymax;	/* range of data */
  int xcol,ycol,xerr,yerr;	/* column numbers */
  int xcolmax, ycolmax;		/* upper range of data column number */
  int ncol;			/* number of data columns in file */
  int connect;			/* points plotted with lines? */
};

typedef struct finfo XYFileInfo;


/* Function prototypes */

int XYintscan(char *s, int *i);
int XYfloatscan(char *s, double *f);
int XYStartStream(XYFileInfo *fi);
int XYReadData(XYFileInfo *fi, double *data);
int XYReadStream(XYFileInfo *fi,
  double *x, double *y, double *xerr, double *yerr);
int XYScanRange(XYFileInfo *fi);
char *XYFormat(double a);

Spline1D *ReadSplineXY(char *fn, int xc, int yc);

/* private functions */
#ifdef XYPLOT_SOURCE
void MakeTicks(FILE *fout, int n, double min, double stp, double max, char c);
char *DefaultLabel(int err, int col);
void fixpoint(double *x, double *y, double m);
void ScanCol(char *s, int *data, int *datamax, int *error);
int finsert(FILE *f, char *fn);
void intscan(char *s, int *i);
int floatscan(char *s, double *f);
char *GetTime(void);
int ticker(double min, double max, int ntick, int lock, 
	double *umin, double *ustp, double *umax);
void putUnix(FILE *f, double x, double y);
void setUnix(FILE *f, double x0, double x1, double y0, double y1);

void DumpPlot(FILE *fout, XYFileInfo *fi, char c);
void DumpData(FILE *f, XYFileInfo *fi);
void DumpUnix(FILE *f, XYFileInfo *fi);

extern char *xyplot_path;
#endif /* XYPLOT_SOURCE */

#endif /* _XYPLOT_H_ */
